# Copyright 2020 Bateared Collie
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
Base functinoality
'''
"""
Functions that will all take a file pointer and the sample count and return a
NumPy array with the unpacked values.
"""
# from __future__ import (absolute_import, division, print_function,
#                         unicode_literals)
# from future.builtins import *  # NOQA

import sys

import numpy as np


LOG2 = 0.3010299956639812
# Get the system byte order.
BYTEORDER = sys.byteorder
if BYTEORDER == 'little':
    BYTEORDER = '<'
else:
    BYTEORDER = '>'


class WrongDtypeException(Exception):
    pass


def pack_4byte_ibm(file, data, endian='>'):
    """
    Packs 4 byte IBM floating points. This will only work if the host system
    internally uses little endian byte orders.
    """
    # Check the dtype and raise exception otherwise!
    if data.dtype != np.float64 and data.dtype != np.float32:
        raise WrongDtypeException
    # Calculate the values. The theory is explained in
    # https://www.codeproject.com/KB/applications/libnumber.aspx

    # Calculate the signs.
    signs = np.empty(len(data), dtype=np.uint8)
    temp_signs = np.sign(data)
    # Negative numbers are encoded as sign bit 1, positive ones as bit 0.
    signs[temp_signs == 1] = 0
    signs[temp_signs == -1] = 128

    # Make absolute values.
    data = np.abs(data)

    # Store the zeros and add an offset for numerical stability,
    # they will be set to zero later on again
    zeros = np.where(data == 0.0)
    data[zeros] += 1e-32

    # Calculate the exponent for the IBM data format.
    exponent = ((np.log10(data) / LOG2) * 0.25 + 65).astype(np.uint32)

    # Now calculate the fraction using single precision.
    fraction = np.require(
        data, np.float32) / (16.0 ** (np.require(exponent, np.float32) - 64))

    # Normalization.
    while True:
        # Find numbers smaller than 1/16 but not zero.
        non_normalized = np.where(np.where(fraction, fraction, 1) < 0.0625)[0]
        if len(non_normalized) == 0:
            break
        fraction[non_normalized] *= 16
        exponent[non_normalized] -= 1

    # If the fraction is one, change it to 1/16 and increase the exponent by
    # one.
    ones = np.where(fraction == 1.0)
    fraction[ones] = 0.0625
    exponent[ones] += 1

    # Times 2^24 to be able to get a long.
    fraction *= 16777216.0
    # Convert to unsigned long.
    fraction = np.require(fraction, np.uint64)

    # Use 8 bit integers to be able to store every byte separately.
    new_data = np.zeros(4 * len(data), np.uint8)

    # The first bit is the sign and the following 7 are the exponent.
    byte_0 = np.require(signs + exponent, np.uint8)
    # All following 24 bit are the fraction.
    byte_1 = np.require(np.right_shift(np.bitwise_and(fraction, 0x00ff0000),
                                       16), np.uint8)
    byte_2 = np.require(np.right_shift(np.bitwise_and(fraction, 0x0000ff00),
                                       8), np.uint8)
    byte_3 = np.require(np.bitwise_and(fraction, 0x000000ff), np.uint8)

    # Depending on the endianness store the data different.
    # big endian.
    if endian == '>':
        new_data[0::4] = byte_0
        new_data[1::4] = byte_1
        new_data[2::4] = byte_2
        new_data[3::4] = byte_3
    # little endian>
    elif endian == '<':
        new_data[0::4] = byte_3
        new_data[1::4] = byte_2
        new_data[2::4] = byte_1
        new_data[3::4] = byte_0
    # Should not happen.
    else:
        raise Exception
    # Write the zeros again.
    new_data.dtype = np.uint32
    new_data[zeros] = 0
    # Write to file.
    #file.write(new_data.tostring())
    file.write(new_data.tobytes())


def pack_4byte_integer(file, data, endian='>'):
    """
    Packs 4 byte integers.
    """
    # Check the dtype and raise exception otherwise!
    if data.dtype != np.int32:
        raise WrongDtypeException
    # Swap the byte order if necessary.
    if BYTEORDER != endian:
        data = data.byteswap()
    # Write the file.
    file.write(data.tostring())


def pack_2byte_integer(file, data, endian='>'):
    """
    Packs 2 byte integers.
    """
    # Check the dtype and raise exception otherwise!
    if data.dtype != np.int16:
        raise WrongDtypeException
    # Swap the byte order if necessary.
    if BYTEORDER != endian:
        data = data.byteswap()
    # Write the file.
    file.write(data.tostring())


def pack_4byte_fixed_point(file, data, endian='>'):
    raise NotImplementedError


def pack_4byte_ieee(file, data, endian='>'):
    """
    Packs 4 byte IEEE floating points.
    """
    # Check the dtype and raise exception otherwise!
    if data.dtype != np.float32:
        raise WrongDtypeException
    # Swap the byte order if necessary.
    if BYTEORDER != endian:
        data = data.byteswap()
    # Write the file.
    file.write(data.tobytes())


def pack_1byte_integer(file, data, endian='>'):
    raise NotImplementedError



# -*- coding: utf-8 -*-
"""
The obspy.io.seg2 test suite.
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from future.builtins import *  # NOQA

import gzip
import os
import unittest
import warnings

import numpy as np

from obspy import read


TRACE2_HEADER = {'ACQUISITION_DATE': '07/JAN/2013',
                 'ACQUISITION_DATE_UTC': '07/JAN/2013',
                 'ACQUISITION_TIME': '10:30:41',
                 'ACQUISITION_TIME_MICROSECONDS': '0',
                 'ACQUISITION_TIME_MICROSECONDS_UTC': '0',
                 'ACQUISITION_TIME_UTC': '09:30:41',
                 'APPLIED_STANDARD': '0',
                 'BATTERY_LEVEL': '99 0 30.35 4.123',
                 'CHANNEL_NUMBER': '1',
                 'CLIENT': 'CLIENT',
                 'CLOCK_SOURCE': '1',
                 'COMPANY': 'COMPANY',
                 'DESCALING_FACTOR': '2.17378e-05',
                 'DEVICE_NAME': 'VIPA 15',
                 'DEVICE_SERIAL_NO': '01-0000143912a3',
                 'DIFFERENCE_TO_REAL_TIME': '0.000000',
                 'FIRMWARE_VERSION': '0.0.125',
                 'FIRST_SAMPLE_NO': '8000',
                 'HIGH_CUT_FILTER': '0 0',
                 'INSTRUMENT': 'DMT_VIPA_01-0000143912a3',
                 'LAST_SAMPLE_NO': '9999',
                 'LOCATION': 'LOCATION',
                 'LOW_CUT_FILTER': '10.000000 12.000000',
                 'NETWORK_NAME': 'BANK',
                 'NOTE': ['Comment'],
                 'OBSERVER': 'OBSERVER',
                 'REAL_TIME_AVAILABLE': 'FALSE',
                 'REGISTRATION_DIRECTION': 'X',
                 'SAMPLE_INTERVAL': '0.00100000',
                 'SCALE_UNIT': 'mm/s',
                 'SECONDS_SINCE_LAST_GPS_SYNC': '-1',
                 'SENSOR_CALIB_DATE': '21/8/12',
                 'SENSOR_FC': '4.500000',
                 'SENSOR_TYPE_ID': '1',
                 'SENSOR_TYPE_NAME': 'DMT-3D/DIN',
                 'STATION_CODE': 'BA1',
                 'STATION_NAME': 'DMT-BANK',
                 'SYSTEM_T0': '0',
                 'TIME_ZONE': 'CET',
                 'TRACE_TYPE': 'SEISMIC_DATA',
                 'TRIGGER_LEVEL': '2.00000000',
                 'TRIGGER_SAMPLE_NO': '8000',
                 'TRIGGER_SOURCE': '0',
                 'UNITS': 'METERS'}
TRACE3_HEADER = {
    'ACQUISITION_DATE': '7/MAR/2018',
    'ACQUISITION_TIME': '3:12:45',
    'INSTRUMENT': 'GEOMETRICS SmartSeis 0000',
    'TRACE_SORT': 'AS_ACQUIRED',
    'UNITS': 'METERS',
    'NOTE': ['DISPLAY_SCALE 48'],
    'CHANNEL_NUMBER': '1',
    'DELAY': '-0.010',
    'DESCALING_FACTOR': '0.001199',
    'LINE_ID': '00-00',
    'LOW_CUT_FILTER': '0 0',
    'NOTCH_FREQUENCY': '0',
    'RAW_RECORD': '1068.DAT',
    'RECEIVER_LOCATION': '1004.00',
    'SAMPLE_INTERVAL': '0.000125',
    'SKEW': '-0.00001796',
    'SOURCE_LOCATION': '1000.00',
    'STACK': '8',
}


class SEG2TestCase(unittest.TestCase):
    """
    Test cases for SEG2 reading.
    """
    def setUp(self):
        # directory where the test files are located
        self.dir = os.path.dirname(__file__)
        self.path = os.path.join(self.dir, 'data')

    def test_read_data_format_2(self):
        """
        Test reading a SEG2 data format code 2 file (int32).
        """
        basename = os.path.join(self.path,
                                '20130107_103041000.CET.3c.cont.0')
        # read SEG2 data (in counts, int32)
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')
            st = read(basename + ".seg2.gz")
        self.assertEqual(len(w), 1)
        self.assertIn('custom', str(w[0]))
        # read reference ASCII data (in micrometer/s)
        with gzip.open(basename + ".DAT.gz", 'rb') as f:
            results = np.loadtxt(f).T
        # test all three components
        for tr, result in zip(st, results):
            # convert raw data to micrometer/s (descaling goes to mm/s)
            scaled_data = tr.data * float(tr.stats.seg2.DESCALING_FACTOR) * 1e3
            self.assertTrue(np.allclose(scaled_data, result, rtol=1e-7,
                                        atol=1e-7))
        # test seg2 specific header values
        # (trace headers include SEG2 file header)
        self.assertEqual(st[0].stats.seg2, TRACE2_HEADER)

    def test_read_data_format_3(self):
        """
        Test reading a SEG2 data format code 3 file (20-bit floating point).
        """
        basename = os.path.join(self.path,
                                '20180307_031245000.0')
        # read SEG2 data (in counts, int32)
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')
            st = read(basename + ".seg2")
        self.assertEqual(len(w), 2)
        self.assertIn('custom', str(w[1]))
        # read reference ASCII data
        with gzip.open(basename + '.DAT.gz', 'rb') as f:
            results = np.loadtxt(f, ndmin=2).T
        for tr, result in zip(st, results):
            # convert raw data to unit'd
            scaled_data = tr.data * tr.stats.calib
            np.testing.assert_allclose(scaled_data, result,
                                       rtol=1e-7, atol=1e-7)
        # test seg2 specific header values
        # (trace headers include SEG2 file header)
        self.assertEqual(st[0].stats.seg2, TRACE3_HEADER)


def suite():
    return unittest.makeSuite(SEG2TestCase, 'test')


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



# Copyright 2020 Bateared Collie
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
Attribute dictionary
'''
# from __future__ import (absolute_import, division, print_function,
#                         unicode_literals)
# from future.builtins import *  # NOQA @UnusedWildImport

import copy
import warnings

from .. import compatibility


class AttribDict(compatibility.collections_abc.MutableMapping):
    """
    A class which behaves like a dictionary.

    :type data: dict, optional
    :param data: Dictionary with initial keywords.

    .. rubric:: Basic Usage

    You may use the following syntax to change or access data in this class.

    >>> stats = AttribDict()
    >>> stats.network = 'BW'
    >>> stats['station'] = 'ROTZ'
    >>> print(stats.get('network'))
    BW
    >>> print(stats['network'])
    BW
    >>> print(stats.station)
    ROTZ
    >>> x = stats.keys()
    >>> x = sorted(x)
    >>> print(x[0], x[1])
    network station
    """
    defaults = {}
    readonly = []
    warn_on_non_default_key = False
    do_not_warn_on = []
    _types = {}

    def __init__(self, *args, **kwargs):
        """
        An AttribDict can be initialized in two ways. It can be given an
        existing dictionary as a simple argument or alternatively all keyword
        arguments will become (key, value) pairs.

        >>> attrib_dict_1 = AttribDict({"a":1, "b":2})
        >>> attrib_dict_2 = AttribDict(a=1, b=2)
        >>> attrib_dict_1  #doctest: +SKIP
        AttribDict({'a': 1, 'b': 2})
        >>> assert(attrib_dict_1 == attrib_dict_2)
        """
        # set default values directly
        self.__dict__.update(self.defaults)
        # use overwritable update method to set arguments
        self.update(dict(*args, **kwargs))

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self.__dict__)

    def __getitem__(self, name, default=None):
        try:
            return self.__dict__[name]
        except KeyError:
            # check if we got any default value given at class level
            if name in self.defaults:
                return self.defaults[name]
            # if both are missing check for a given default value
            if default is None:
                raise
            return default

    def __setitem__(self, key, value):
        if key in self.readonly:
            msg = 'Attribute "%s" in %s object is read only!'
            raise AttributeError(msg % (key, self.__class__.__name__))
        if self.warn_on_non_default_key and key not in self.defaults:
            # issue warning if not a default key
            # (and not in the list of exceptions)
            if key in self.do_not_warn_on:
                pass
            else:
                msg = ('Setting attribute "{}" which is not a default '
                       'attribute ("{}").').format(
                    key, '", "'.join(self.defaults.keys()))
                warnings.warn(msg)
        # Type checking/warnings
        if key in self._types and not isinstance(value, self._types[key]):
            value = self._cast_type(key, value)

        mapping_instance = isinstance(value,
                                      compatibility.collections_abc.Mapping)
        attr_dict_instance = isinstance(value, AttribDict)
        if mapping_instance and not attr_dict_instance:
            self.__dict__[key] = AttribDict(value)
        else:
            self.__dict__[key] = value

    def __delitem__(self, name):
        del self.__dict__[name]

    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, adict):
        # set default values
        self.__dict__.update(self.defaults)
        # update with pickle dictionary
        self.update(adict)

    def __getattr__(self, name, default=None):
        """
        Py3k hasattr() expects an AttributeError no KeyError to be
        raised if the attribute is not found.
        """
        try:
            return self.__getitem__(name, default)
        except KeyError as e:
            raise AttributeError(e.args[0])

    __setattr__ = __setitem__
    __delattr__ = __delitem__

    def copy(self):
        return copy.deepcopy(self)

    def __deepcopy__(self, *args, **kwargs):  # @UnusedVariable
        ad = self.__class__()
        ad.update(copy.deepcopy(self.__dict__))
        return ad

    def update(self, adict={}):
        for (key, value) in adict.items():
            if key in self.readonly:
                continue
            self.__setitem__(key, value)

    def _pretty_str(self, priorized_keys=[], min_label_length=16):
        """
        Return better readable string representation of AttribDict object.

        :type priorized_keys: list of str, optional
        :param priorized_keys: Keywords of current AttribDict which will be
            shown before all other keywords. Those keywords must exists
            otherwise an exception will be raised. Defaults to empty list.
        :type min_label_length: int, optional
        :param min_label_length: Minimum label length for keywords. Defaults
            to ``16``.
        :return: String representation of current AttribDict object.
        """
        keys = list(self.keys())
        # determine longest key name for alignment of all items
        try:
            i = max(max([len(k) for k in keys]), min_label_length)
        except ValueError:
            # no keys
            return ""
        pattern = "%%%ds: %%s" % (i)
        # check if keys exist
        other_keys = [k for k in keys if k not in priorized_keys]
        # priorized keys first + all other keys
        keys = priorized_keys + sorted(other_keys)
        head = [pattern % (k, self.__dict__[k]) for k in keys]
        return "\n".join(head)

    def _cast_type(self, key, value):
        """
        Cast type of value to type required in _types dict.

        :type key: str
        :param key: The key from __setattr__.
        :param value: The value being set to key.
        :return: value cast to correct type.
        """
        typ = self._types[key]
        new_type = (
            typ[0] if isinstance(typ, compatibility.collections_abc.Sequence)
            else typ)
        msg = ('Attribute "%s" must be of type %s, not %s. Attempting to '
               'cast %s to %s') % (key, typ, type(value), value, new_type)
        warnings.warn(msg)
        return new_type(value)

    def __iter__(self):
        return iter(self.__dict__)

    def __len__(self):
        return len(self.__dict__)




# Copyright 2020 Bateared Collie
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
Decorator
'''
# from __future__ import (absolute_import, division, print_function,
#                         unicode_literals)
# from future.builtins import *  # NOQA
# 
# import functools
import inspect
# import os
import re
# import socket
# import tarfile
# import unittest
# import warnings
# import zipfile
# 
# import numpy as np
from decorator import decorator
# from future.utils import PY2, native_str

from obsln.core.futureutils import native_str

# 
from obsln.core.util import get_example_file
# from obspy.core.util.base import NamedTemporaryFile
# from obspy.core.util.deprecation_helpers import ObsPyDeprecationWarning
# 
# 
# def deprecated(warning_msg=None):
#     """
#     This is a decorator which can be used to mark functions as deprecated.
# 
#     .. note::
#         Actually, this is not a decorator itself but a decorator factory,
#         returning the correct decorator for the specified options. It can be
#         used just like a decorator.
# 
#     It will result in a warning being emitted when the function is used.
#     """
#     @decorator
#     def _deprecated(func, *args, **kwargs):
#         if 'deprecated' in str(func.__doc__).lower():
#             msg = func.__doc__
#         elif warning_msg:
#             msg = warning_msg
#             if PY2 and inspect.ismethod(func):
#                 func.im_func.__doc__ = warning_msg
#             else:
#                 func.__doc__ = warning_msg
#         else:
#             msg = "Call to deprecated function %s." % func.__name__
#         warnings.warn(msg, category=ObsPyDeprecationWarning, stacklevel=3)
#         return func(*args, **kwargs)
#     return _deprecated
# 
# 
# def deprecated_keywords(keywords):
#     """
#     Decorator for marking keywords as deprecated.
# 
#     .. note::
#         Actually, this is not a decorator itself but a decorator factory,
#         returning the correct decorator for the specified options. It can be
#         used just like a decorator.
# 
#     :type keywords: dict
#     :param keywords: old/new keyword names as key/value pairs.
#     """
#     def fdec(func):
#         fname = func.__name__
#         msg = "Deprecated keyword %s in %s() call - please use %s instead."
#         msg2 = "Deprecated keyword %s in %s() call - ignoring."
#         msg3 = ("Conflicting deprecated keywords (%s) in %s() call"
#                 " - please use new '%s' keyword instead.")
# 
#         @functools.wraps(func)
#         def echo_func(*args, **kwargs):
#             # check if multiple deprecated keywords get mapped to the same new
#             # keyword
#             new_keyword_appearance_counts = dict.fromkeys(keywords.values(), 0)
#             for key, new_key in keywords.items():
#                 if key in kwargs:
#                     new_keyword_appearance_counts[new_key] += 1
#             for key_ in keywords.values():
#                 # ignore `None` as new value, it means that no mapping is
#                 # happening..
#                 if key_ is None:
#                     continue
#                 if new_keyword_appearance_counts[key_] > 1:
#                     conflicting_keys = ", ".join(
#                         [old_key for old_key, new_key in keywords.items()
#                          if new_key == key_])
#                     raise Exception(msg3 % (conflicting_keys, fname, new_key))
#             # map deprecated keywords to new keywords
#             for kw in kwargs.keys():
#                 if kw in keywords:
#                     nkw = keywords[kw]
#                     if nkw is None:
#                         warnings.warn(msg2 % (kw, fname),
#                                       category=ObsPyDeprecationWarning,
#                                       stacklevel=3)
#                     else:
#                         warnings.warn(msg % (kw, fname, nkw),
#                                       category=ObsPyDeprecationWarning,
#                                       stacklevel=3)
#                         kwargs[nkw] = kwargs[kw]
#                     del(kwargs[kw])
#             return func(*args, **kwargs)
#         return echo_func
# 
#     return fdec
# 
# 
# @decorator
# def skip_on_network_error(func, *args, **kwargs):
#     """
#     Decorator for unittest to mark test routines that fail with certain network
#     errors (e.g. timeouts) as "skipped" rather than "Error".
#     """
#     try:
#         return func(*args, **kwargs)
#     ###################################################
#     # add more except clauses like this to add other
#     # network errors that should be skipped
#     except socket.timeout as e:
#         if str(e) == "timed out":
#             raise unittest.SkipTest(str(e))
#     ###################################################
#     except socket.error as e:
#         if str(e) == "[Errno 110] Connection timed out":
#             raise unittest.SkipTest(str(e))
#     # general except to be able to generally reraise
#     except Exception:
#         raise
# 
# 
# @decorator
# def uncompress_file(func, filename, *args, **kwargs):
#     """
#     Decorator used for temporary uncompressing file if .gz or .bz2 archive.
#     """
#     if not kwargs.get('check_compression', True):
#         return func(filename, *args, **kwargs)
#     if not isinstance(filename, (str, native_str)):
#         return func(filename, *args, **kwargs)
#     elif not os.path.exists(filename):
#         msg = "File not found '%s'" % (filename)
#         raise IOError(msg)
#     # check if we got a compressed file or archive
#     obj_list = []
#     if tarfile.is_tarfile(filename):
#         try:
#             # reading with transparent compression
#             with tarfile.open(filename, 'r|*') as tar:
#                 for tarinfo in tar:
#                     # only handle regular files
#                     if not tarinfo.isfile():
#                         continue
#                     data = tar.extractfile(tarinfo).read()
#                     # Skip empty files - we don't need them no matter what
#                     # and it guards against rare cases where waveforms files
#                     # are also slightly valid tar-files.
#                     if not data:
#                         continue
#                     obj_list.append(data)
#         except Exception:
#             pass
#     elif zipfile.is_zipfile(filename):
#         try:
#             zip = zipfile.ZipFile(filename)
#             obj_list = [zip.read(name) for name in zip.namelist()]
#         except Exception:
#             pass
#     elif filename.endswith('.bz2'):
#         # bz2 module
#         try:
#             import bz2
#             with open(filename, 'rb') as fp:
#                 obj_list.append(bz2.decompress(fp.read()))
#         except Exception:
#             pass
#     elif filename.endswith('.gz'):
#         # gzip module
#         try:
#             import gzip
#             with gzip.open(filename, 'rb') as fp:
#                 obj_list.append(fp.read())
#         except Exception:
#             pass
#     # handle results
#     if obj_list:
#         # write results to temporary files
#         result = None
#         for obj in obj_list:
#             with NamedTemporaryFile() as tempfile:
#                 tempfile._fileobj.write(obj)
#                 stream = func(tempfile.name, *args, **kwargs)
#                 # just add other stream objects to first stream
#                 if result is None:
#                     result = stream
#                 else:
#                     result += stream
#     else:
#         # no compressions
#         result = func(filename, *args, **kwargs)
#     return result
# 
# 
# @decorator
# def raise_if_masked(func, *args, **kwargs):
#     """
#     Raises if the first argument (self in case of methods) is a Trace with
#     masked values or a Stream containing a Trace with masked values.
#     """
#     arrays = []
#     # first arg seems to be a Stream
#     if hasattr(args[0], "traces"):
#         arrays = [tr.data for tr in args[0]]
#     # first arg seems to be a Trace
#     if hasattr(args[0], "data") and isinstance(args[0].data, np.ndarray):
#         arrays = [args[0].data]
#     for arr in arrays:
#         if np.ma.is_masked(arr):
#             msg = "Trace with masked values found. This is not " + \
#                   "supported for this operation. Try the split() " + \
#                   "method on Trace/Stream to produce a Stream with " + \
#                   "unmasked Traces."
#             raise NotImplementedError(msg)
#     return func(*args, **kwargs)
# 
# 
# @decorator
# def skip_if_no_data(func, *args, **kwargs):
#     """
#     Does nothing if the first argument (self in case of methods) is a Trace
#     with no data in it.
#     """
#     if not args[0]:
#         return
#     return func(*args, **kwargs)
# 
 
def map_example_filename(arg_kwarg_name):
    """
    Decorator that replaces "/path/to/filename" patterns in the arg or kwarg
    of the specified name with the correct file path. If the pattern is not
    encountered nothing is done.
 
    .. note::
        Actually, this is not a decorator itself but a decorator factory,
        returning the correct decorator for the specified options. It can be
        used just like a decorator.
 
    :type arg_kwarg_name: str
    :param arg_kwarg_name: name of the arg/kwarg that should be (tried) to map
    """
    @decorator
    def _map_example_filename(func, *args, **kwargs):
        prefix = '/path/to/'
        # check kwargs
        if arg_kwarg_name in kwargs:
            if isinstance(kwargs[arg_kwarg_name], (str, native_str)):
                if re.match(prefix, kwargs[arg_kwarg_name]):
                    try:
                        kwargs[arg_kwarg_name] = \
                            get_example_file(kwargs[arg_kwarg_name][9:])
                    # file not found by get_example_file:
                    except IOError:
                        pass
        # check args
        else:
            try:
                inspected_args = [
                    p.name
                    for p in inspect.signature(func).parameters.values()
                ]
            except AttributeError:
                inspected_args = inspect.getargspec(func).args
            try:
                ind = inspected_args.index(arg_kwarg_name)
            except ValueError:
                pass
            else:
                if ind < len(args) and isinstance(args[ind], (str,
                                                              native_str)):
                    # need to check length of args from inspect
                    if re.match(prefix, args[ind]):
                        try:
                            args = list(args)
                            args[ind] = get_example_file(args[ind][9:])
                            args = tuple(args)
                        # file not found by get_example_file:
                        except IOError:
                            pass
        return func(*args, **kwargs)
    return _map_example_filename
# 
# 
# if __name__ == '__main__':
#     import doctest
#     doctest.testmod(exclude_empty=True)



# -*- coding: utf-8 -*-
"""
Headers for obspy.io.seg2.

:copyright:
    Lion Krischer (krischer@geophysik.uni-muenchen.de), 2011
:license:
    GNU Lesser General Public License, Version 3
    (https://www.gnu.org/copyleft/lesser.html)
"""
# from __future__ import (absolute_import, division, print_function,
#                         unicode_literals)
# from future.builtins import *  # NOQA


MONTHS = {'jan': 1,
          'feb': 2,
          'mar': 3,
          'apr': 4,
          'may': 5,
          'jun': 6,
          'jul': 7,
          'aug': 8,
          'sep': 9,
          'oct': 10,
          'nov': 11,
          'dec': 12,
          '1': 1,
          '2': 2,
          '3': 3,
          '4': 4,
          '5': 5,
          '6': 6,
          '7': 7,
          '8': 8,
          '9': 9,
          '10': 10,
          '11': 11,
          '12': 12,
          '01': 1,
          '02': 2,
          '03': 3,
          '04': 4,
          '05': 5,
          '06': 6,
          '07': 7,
          '08': 8,
          '09': 9
          }



# Copyright 2020 Bateared Collie
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
# 

from struct import pack, unpack

from obsln.core.util.libnames import _load_cdll


# Import shared libsegy
clibsegy = _load_cdll("segy")


def unpack_header_value(endian, packed_value, length, special_format):
    """
    Unpacks a single value.
    """
    # Use special format if necessary.
    if special_format:
        fmt = ('%s%s' % (endian, special_format)).encode('ascii', 'strict')
        return unpack(fmt, packed_value)[0]
    # Unpack according to different lengths.
    elif length == 2:
        format = ('%sh' % endian).encode('ascii', 'strict')
        return unpack(format, packed_value)[0]
    # Update: Seems to be correct. Two's complement integers seem to be
    # the common way to store integer values.
    elif length == 4:
        format = ('%si' % endian).encode('ascii', 'strict')
        return unpack(format, packed_value)[0]
    # The unassigned field. Since it is unclear how this field is
    # encoded it will just be stored as a string.
    elif length == 8:
        return packed_value
    # Should not happen
    else:
        raise Exception


def _pack_attribute_nicer_exception(obj, name, format):
    """
    packs obj.name with the given format but raises a nicer error message.
    """
    x = getattr(obj, name)
    try:
        return pack(format, x)
    except Exception as e:
        msg = ("Failed to pack header value `%s` (%s) with format `%s` due "
               "to: `%s`")
        try:
            format = format.decode()
        except AttributeError:
            pass
        raise ValueError(msg % (name, str(x), format, e.args[0]))



# Copyright 2020 Bateared Collie
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
Base functinoality
'''
"""
Library name handling for ObsPy.

:copyright:
    The ObsPy Development Team (devs@obspy.org)
:license:
    GNU Lesser General Public License, Version 3
    (https://www.gnu.org/copyleft/lesser.html)
"""
# NO IMPORTS FROM OBSPY OR FUTURE IN THIS FILE! (file gets used at
# installation time)
import ctypes
import os
import platform
import re
import warnings
from distutils import sysconfig


def cleanse_pymodule_filename(filename):
    """
    Replace all characters not allowed in Python module names in filename with
    "_".

    See bug report:
     - https://stackoverflow.com/q/21853678
     - See #755

    See also:
     - https://stackoverflow.com/q/7552311
     - https://docs.python.org/3/reference/lexical_analysis.html#identifiers

    >>> print(cleanse_pymodule_filename("0blup-bli.554_3!32"))
    _blup_bli_554_3_32
    """
    filename = re.sub(r'^[^a-zA-Z_]', "_", filename)
    filename = re.sub(r'[^a-zA-Z0-9_]', "_", filename)
    return filename


def _get_lib_name(lib, add_extension_suffix):
    """
    Helper function to get an architecture and Python version specific library
    filename.

    :type add_extension_suffix: bool
    :param add_extension_suffix: NumPy distutils adds a suffix to
        the filename we specify to build internally (as specified by Python
        builtin `sysconfig.get_config_var("EXT_SUFFIX")`. So when loading the
        file we have to add this suffix, but not during building.
    """
    # our custom defined part of the extension file name
    libname = "lib%s_%s_%s_py%s" % (
        lib, platform.system(), platform.architecture()[0],
        ''.join([str(i) for i in platform.python_version_tuple()[:2]]))
    libname = cleanse_pymodule_filename(libname)
    # NumPy distutils adds extension suffix by itself during build (#771, #755)
    if add_extension_suffix:
        # append any extension suffix defined by Python for current platform
        ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
        # in principle "EXT_SUFFIX" is what we want.
        # "SO" seems to be deprecated on newer python
        # but: older python seems to have empty "EXT_SUFFIX", so we fall back
        if not ext_suffix:
            try:
                ext_suffix = sysconfig.get_config_var("SO")
            except Exception as e:
                msg = ("Empty 'EXT_SUFFIX' encountered while building CDLL "
                       "filename and fallback to 'SO' variable failed "
                       "(%s)." % str(e))
                warnings.warn(msg)
                pass
        if ext_suffix:
            libname = libname + ext_suffix
    return libname


def _load_cdll(name):
    '''
    Adding this function so it does not fail when runnign from source 
    '''
    
    try:
        return _load_cdll4Real(name,useBuild=False)
    
    except ImportError:
        return _load_cdll4Real(name,useBuild=True)

    

def _load_cdll4Real(name,useBuild=False):
    """
    Helper function to load a shared library built during ObsPy installation
    with ctypes.

    :type name: str
    :param name: Name of the library to load (e.g. 'mseed').
    :rtype: :class:`ctypes.CDLL`
    """
    

    # our custom defined part of the extension file name
    libname = _get_lib_name(name, add_extension_suffix=True)

    if useBuild==True: 
        libdir = os.path.join(os.path.dirname(__file__), os.pardir, os.pardir,os.pardir,
                              'build/lib.linux-x86_64-3.8/obsln/lib/')  
    else:
        libdir = os.path.join(os.path.dirname(__file__), os.pardir, os.pardir,'lib')        


    libpath = os.path.join(libdir, libname)

    # resolve parent directory '../' for windows
    libpath = os.path.normpath(libpath)
    libpath = os.path.abspath(libpath)
    libpath = str(libpath)

    try:
        cdll = ctypes.CDLL(libpath)
    except Exception as e:
        import textwrap
        dirlisting = textwrap.wrap(
            ', '.join(sorted(os.path.dirname(libpath))))
        msg = ['Could not load shared library "%s"' % libname,
               'Path: %s' % libpath,
               'Current directory: %s' % os.path.abspath(os.curdir),
               'ctypes error message: %s' % str(e),
               'Directory listing of lib directory:',
               '  ',
               ]
        msg = '\n  '.join(msg)
        msg = msg + '\n    '.join(dirlisting)
        raise ImportError(msg)
    return cdll


if __name__ == '__main__':
    import doctest
    doctest.testmod(exclude_empty=True)



# Author: Kit Chambers
# Contact: kit.chambers@motionsignaltechnologies.com
# 
# Motion Signal Technologies Ltd.
# All rights reserved
# 
# The contents of this file are considered proprietary and usage or
# reproduction without prior authorization is strictly prohibited.
'''
ObsLn - Provides some of the basic functionality from ObsPy without many of the dependencies and size. For full ObsPy functionality please see the ObsPy project on GitHub [here](https://github.com/obspy)

'''

try:
    import setuptools
except ImportError:
    pass

try:
    import numpy 
except ImportError:
    msg = ("No module named numpy. "
           "Please install numpy first, it is needed before installing ObsLn.")
    raise ImportError(msg)

import fnmatch
import glob
import inspect
import os
import sys
import platform
from distutils.util import change_root

from numpy.distutils.core import setup
from numpy.distutils.ccompiler import get_default_compiler
from numpy.distutils.command.build import build
from numpy.distutils.command.install import install
from numpy.distutils.exec_command import exec_command, find_executable
from numpy.distutils.misc_util import Configuration

SETUP_DIRECTORY = os.path.dirname(os.path.abspath(inspect.getfile(
    inspect.currentframe())))

UTIL_PATH = os.path.join(SETUP_DIRECTORY, "obsln", "core", "util")
sys.path.insert(0, UTIL_PATH)
#from version import get_git_version  # @UnresolvedImport
from libnames import _get_lib_name  # @UnresolvedImport
sys.path.pop(0)

LOCAL_PATH = os.path.join(SETUP_DIRECTORY, "setup.py")
DOCSTRING = __doc__.split("\n")

INSTALL_REQUIRES=[]

EXTRAS_REQUIRE={}

ENTRY_POINTS = {
#     'console_scripts': [
#         'obspy-flinn-engdahl = obspy.scripts.flinnengdahl:main',
#         'obspy-runtests = obspy.scripts.runtests:main',
#         'obspy-reftek-rescue = obspy.scripts.reftekrescue:main',
#         'obspy-print = obspy.scripts._print:main',
#         'obspy-sds-report = obspy.scripts.sds_html_report:main',
#         'obspy-indexer = obspy.db.scripts.indexer:main',
#         'obspy-scan = obspy.imaging.scripts.scan:main',
#         'obspy-plot = obspy.imaging.scripts.plot:main',
#         'obspy-mopad = obspy.imaging.scripts.mopad:main',
#         'obspy-mseed-recordanalyzer = '
#         'obspy.io.mseed.scripts.recordanalyzer:main',
#         'obspy-dataless2xseed = obspy.io.xseed.scripts.dataless2xseed:main',
#         'obspy-xseed2dataless = obspy.io.xseed.scripts.xseed2dataless:main',
#         'obspy-dataless2resp = obspy.io.xseed.scripts.dataless2resp:main',
#         ],
    'obsln.plugin.waveform': [
#         'TSPAIR = obspy.io.ascii.core',
#         'SLIST = obspy.io.ascii.core',
        'PICKLE = obsln.core.stream',
#         'CSS = obspy.io.css.core',
#         'WIN = obspy.io.win.core',
#         'KINEMETRICS_EVT = obspy.io.kinemetrics.core',
#         'GSE1 = obspy.io.gse2.core',
#         'GSE2 = obspy.io.gse2.core',
#         'MSEED = obspy.io.mseed.core',
#         'NNSA_KB_CORE = obspy.io.css.core',
#         'PDAS = obspy.io.pdas.core',
#         'SAC = obspy.io.sac.core',
#         'SACXY = obspy.io.sac.core',
#         'Y = obspy.io.y.core',
        'SEG2 = obsln.io.seg2.seg2',
        'SEGY = obsln.io.segy.core',
        'SU = obsln.io.segy.core',
#         'SEISAN = obspy.io.seisan.core',
#         'Q = obspy.io.sh.core',
#         'SH_ASC = obspy.io.sh.core',
#         'WAV = obspy.io.wav.core',
#         'AH = obspy.io.ah.core',
#         'KNET = obspy.io.nied.knet',
#         'GCF = obspy.io.gcf.core',
#         'REFTEK130 = obspy.io.reftek.core',
#         'RG16 = obspy.io.rg16.core',
#         'DMX = obspy.io.dmx.core',
        ],
#     'obspy.plugin.waveform.TSPAIR': [
#         'isFormat = obspy.io.ascii.core:_is_tspair',
#         'readFormat = obspy.io.ascii.core:_read_tspair',
#         'writeFormat = obspy.io.ascii.core:_write_tspair',
#         ],
#     'obspy.plugin.waveform.SLIST': [
#         'isFormat = obspy.io.ascii.core:_is_slist',
#         'readFormat = obspy.io.ascii.core:_read_slist',
#         'writeFormat = obspy.io.ascii.core:_write_slist',
#         ],
    'obsln.plugin.waveform.PICKLE': [
        'isFormat = obsln.core.stream:_is_pickle',
        'readFormat = obsln.core.stream:_read_pickle',
        'writeFormat = obsln.core.stream:_write_pickle',
        ],
#     'obspy.plugin.waveform.CSS': [
#         'isFormat = obspy.io.css.core:_is_css',
#         'readFormat = obspy.io.css.core:_read_css',
#         ],
#     'obspy.plugin.waveform.NNSA_KB_CORE': [
#         'isFormat = obspy.io.css.core:_is_nnsa_kb_core',
#         'readFormat = obspy.io.css.core:_read_nnsa_kb_core',
#         ],
#     'obspy.plugin.waveform.WIN': [
#         'isFormat = obspy.io.win.core:_is_win',
#         'readFormat = obspy.io.win.core:_read_win',
#         ],
#     'obspy.plugin.waveform.KINEMETRICS_EVT': [
#         'isFormat = obspy.io.kinemetrics.core:is_evt',
#         'readFormat = obspy.io.kinemetrics.core:read_evt',
#         ],
#     'obspy.plugin.waveform.GSE1': [
#         'isFormat = obspy.io.gse2.core:_is_gse1',
#         'readFormat = obspy.io.gse2.core:_read_gse1',
#         ],
#     'obspy.plugin.waveform.GSE2': [
#         'isFormat = obspy.io.gse2.core:_is_gse2',
#         'readFormat = obspy.io.gse2.core:_read_gse2',
#         'writeFormat = obspy.io.gse2.core:_write_gse2',
#         ],
#     'obspy.plugin.waveform.MSEED': [
#         'isFormat = obspy.io.mseed.core:_is_mseed',
#         'readFormat = obspy.io.mseed.core:_read_mseed',
#         'writeFormat = obspy.io.mseed.core:_write_mseed',
#         ],
#     'obspy.plugin.waveform.PDAS': [
#         'isFormat = obspy.io.pdas.core:_is_pdas',
#         'readFormat = obspy.io.pdas.core:_read_pdas',
#         ],
#     'obspy.plugin.waveform.SAC': [
#         'isFormat = obspy.io.sac.core:_is_sac',
#         'readFormat = obspy.io.sac.core:_read_sac',
#         'writeFormat = obspy.io.sac.core:_write_sac',
#         ],
#     'obspy.plugin.waveform.SACXY': [
#         'isFormat = obspy.io.sac.core:_is_sac_xy',
#         'readFormat = obspy.io.sac.core:_read_sac_xy',
#         'writeFormat = obspy.io.sac.core:_write_sac_xy',
#         ],
    'obsln.plugin.waveform.SEG2': [
        'isFormat = obsln.io.seg2.seg2:_is_seg2',
        'readFormat = obsln.io.seg2.seg2:_read_seg2',
        ],
    'obsln.plugin.waveform.SEGY': [
        'isFormat = obsln.io.segy.core:_is_segy',
        'readFormat = obsln.io.segy.core:_read_segy',
        'writeFormat = obsln.io.segy.core:_write_segy',
        ],
    'obsln.plugin.waveform.SU': [
        'isFormat = obsln.io.segy.core:_is_su',
        'readFormat = obsln.io.segy.core:_read_su',
        'writeFormat = obsln.io.segy.core:_write_su',
        ],
#     'obspy.plugin.waveform.SEISAN': [
#         'isFormat = obspy.io.seisan.core:_is_seisan',
#         'readFormat = obspy.io.seisan.core:_read_seisan',
#         ],
#     'obspy.plugin.waveform.Q': [
#         'isFormat = obspy.io.sh.core:_is_q',
#         'readFormat = obspy.io.sh.core:_read_q',
#         'writeFormat = obspy.io.sh.core:_write_q',
#         ],
#     'obspy.plugin.waveform.SH_ASC': [
#         'isFormat = obspy.io.sh.core:_is_asc',
#         'readFormat = obspy.io.sh.core:_read_asc',
#         'writeFormat = obspy.io.sh.core:_write_asc',
#         ],
#     'obspy.plugin.waveform.WAV': [
#         'isFormat = obspy.io.wav.core:_is_wav',
#         'readFormat = obspy.io.wav.core:_read_wav',
#         'writeFormat = obspy.io.wav.core:_write_wav',
#         ],
#     'obspy.plugin.waveform.Y': [
#         'isFormat = obspy.io.y.core:_is_y',
#         'readFormat = obspy.io.y.core:_read_y',
#         ],
#     'obspy.plugin.waveform.AH': [
#         'isFormat = obspy.io.ah.core:_is_ah',
#         'readFormat = obspy.io.ah.core:_read_ah',
#         'writeFormat = obspy.io.ah.core:_write_ah1'
#         ],
#     'obspy.plugin.waveform.KNET': [
#         'isFormat = obspy.io.nied.knet:_is_knet_ascii',
#         'readFormat = obspy.io.nied.knet:_read_knet_ascii',
#         ],
#     'obspy.plugin.waveform.GCF': [
#         'isFormat = obspy.io.gcf.core:_is_gcf',
#         'readFormat = obspy.io.gcf.core:_read_gcf',
#         ],
#     'obspy.plugin.waveform.REFTEK130': [
#         'isFormat = obspy.io.reftek.core:_is_reftek130',
#         'readFormat = obspy.io.reftek.core:_read_reftek130',
#         ],
#     'obspy.plugin.waveform.RG16': [
#         'isFormat = obspy.io.rg16.core:_is_rg16',
#         'readFormat = obspy.io.rg16.core:_read_rg16',
#     ],
#     'obspy.plugin.waveform.DMX': [
#         'isFormat = obspy.io.dmx.core:_is_dmx',
#         'readFormat = obspy.io.dmx.core:_read_dmx',
#     ],
#     'obspy.plugin.event': [
#         'QUAKEML = obspy.io.quakeml.core',
#         'SC3ML = obspy.io.seiscomp.event',
#         'ZMAP = obspy.io.zmap.core',
#         'MCHEDR = obspy.io.pde.mchedr',
#         'JSON = obspy.io.json.core',
#         'NDK = obspy.io.ndk.core',
#         'NLLOC_HYP = obspy.io.nlloc.core',
#         'NLLOC_OBS = obspy.io.nlloc.core',
#         'NORDIC = obspy.io.nordic.core',
#         'CNV = obspy.io.cnv.core',
#         'CMTSOLUTION = obspy.io.cmtsolution.core',
#         'SCARDEC = obspy.io.scardec.core',
#         'SHAPEFILE = obspy.io.shapefile.core',
#         'KML = obspy.io.kml.core',
#         'FNETMT = obspy.io.nied.fnetmt',
#         'GSE2 = obspy.io.gse2.bulletin',
#         'IMS10BULLETIN = obspy.io.iaspei.core',
#         'EVT = obspy.io.sh.evt',
#         'FOCMEC = obspy.io.focmec.core',
#         'HYPODDPHA = obspy.io.hypodd.pha'
#         ],
#     'obspy.plugin.event.QUAKEML': [
#         'isFormat = obspy.io.quakeml.core:_is_quakeml',
#         'readFormat = obspy.io.quakeml.core:_read_quakeml',
#         'writeFormat = obspy.io.quakeml.core:_write_quakeml',
#         ],
#     'obspy.plugin.event.SC3ML': [
#         'isFormat = obspy.io.seiscomp.core:_is_sc3ml',
#         'readFormat = obspy.io.seiscomp.event:_read_sc3ml',
#         'writeFormat = obspy.io.seiscomp.event:_write_sc3ml',
#         ],
#     'obspy.plugin.event.MCHEDR': [
#         'isFormat = obspy.io.pde.mchedr:_is_mchedr',
#         'readFormat = obspy.io.pde.mchedr:_read_mchedr',
#         ],
#     'obspy.plugin.event.JSON': [
#         'writeFormat = obspy.io.json.core:_write_json',
#         ],
#     'obspy.plugin.event.ZMAP': [
#         'isFormat = obspy.io.zmap.core:_is_zmap',
#         'readFormat = obspy.io.zmap.core:_read_zmap',
#         'writeFormat = obspy.io.zmap.core:_write_zmap',
#         ],
#     'obspy.plugin.event.CNV': [
#         'writeFormat = obspy.io.cnv.core:_write_cnv',
#         ],
#     'obspy.plugin.event.NDK': [
#         'isFormat = obspy.io.ndk.core:_is_ndk',
#         'readFormat = obspy.io.ndk.core:_read_ndk',
#         ],
#     'obspy.plugin.event.NLLOC_HYP': [
#         'isFormat = obspy.io.nlloc.core:is_nlloc_hyp',
#         'readFormat = obspy.io.nlloc.core:read_nlloc_hyp',
#         ],
#     'obspy.plugin.event.NLLOC_OBS': [
#         'writeFormat = obspy.io.nlloc.core:write_nlloc_obs',
#         ],
#     'obspy.plugin.event.NORDIC': [
#         'writeFormat = obspy.io.nordic.core:write_select',
#         'readFormat = obspy.io.nordic.core:read_nordic',
#         'isFormat = obspy.io.nordic.core:_is_sfile'
#         ],
#     'obspy.plugin.event.CMTSOLUTION': [
#         'isFormat = obspy.io.cmtsolution.core:_is_cmtsolution',
#         'readFormat = obspy.io.cmtsolution.core:_read_cmtsolution',
#         'writeFormat = obspy.io.cmtsolution.core:_write_cmtsolution'
#         ],
#     'obspy.plugin.event.SCARDEC': [
#         'isFormat = obspy.io.scardec.core:_is_scardec',
#         'readFormat = obspy.io.scardec.core:_read_scardec',
#         'writeFormat = obspy.io.scardec.core:_write_scardec'
#         ],
#     'obspy.plugin.event.FNETMT': [
#         'isFormat = obspy.io.nied.fnetmt:_is_fnetmt_catalog',
#         'readFormat = obspy.io.nied.fnetmt:_read_fnetmt_catalog',
#         ],
#     'obspy.plugin.event.GSE2': [
#         'isFormat = obspy.io.gse2.bulletin:_is_gse2',
#         'readFormat = obspy.io.gse2.bulletin:_read_gse2',
#         ],
#     'obspy.plugin.event.SHAPEFILE': [
#         'writeFormat = obspy.io.shapefile.core:_write_shapefile',
#         ],
#     'obspy.plugin.event.KML': [
#         'writeFormat = obspy.io.kml.core:_write_kml',
#         ],
#     'obspy.plugin.event.IMS10BULLETIN': [
#         'isFormat = obspy.io.iaspei.core:_is_ims10_bulletin',
#         'readFormat = obspy.io.iaspei.core:_read_ims10_bulletin',
#         ],
#     'obspy.plugin.event.EVT': [
#         'isFormat = obspy.io.sh.evt:_is_evt',
#         'readFormat = obspy.io.sh.evt:_read_evt',
#         ],
#     'obspy.plugin.event.FOCMEC': [
#         'isFormat = obspy.io.focmec.core:_is_focmec',
#         'readFormat = obspy.io.focmec.core:_read_focmec',
#         ],
#     'obspy.plugin.event.HYPODDPHA': [
#         'isFormat = obspy.io.hypodd.pha:_is_pha',
#         'readFormat = obspy.io.hypodd.pha:_read_pha',
#         ],
#     'obspy.plugin.inventory': [
#         'STATIONXML = obspy.io.stationxml.core',
#         'INVENTORYXML = obspy.io.arclink.inventory',
#         'SC3ML = obspy.io.seiscomp.inventory',
#         'SACPZ = obspy.io.sac.sacpz',
#         'CSS = obspy.io.css.station',
#         'SHAPEFILE = obspy.io.shapefile.core',
#         'STATIONTXT = obspy.io.stationtxt.core',
#         'KML = obspy.io.kml.core',
#         'SEED = obspy.io.xseed.core',
#         'XSEED = obspy.io.xseed.core',
#         'RESP = obspy.io.xseed.core',
#         ],
#     'obspy.plugin.inventory.STATIONXML': [
#         'isFormat = obspy.io.stationxml.core:_is_stationxml',
#         'readFormat = obspy.io.stationxml.core:_read_stationxml',
#         'writeFormat = obspy.io.stationxml.core:_write_stationxml',
#         ],
#     'obspy.plugin.inventory.INVENTORYXML': [
#         'isFormat = obspy.io.arclink.inventory:_is_inventory_xml',
#         'readFormat = obspy.io.arclink.inventory:_read_inventory_xml',
#         ],
#     'obspy.plugin.inventory.SC3ML': [
#         'isFormat = obspy.io.seiscomp.core:_is_sc3ml',
#         'readFormat = obspy.io.seiscomp.inventory:_read_sc3ml',
#         ],
#     'obspy.plugin.inventory.SACPZ': [
#         'writeFormat = obspy.io.sac.sacpz:_write_sacpz',
#         ],
#     'obspy.plugin.inventory.CSS': [
#         'writeFormat = obspy.io.css.station:_write_css',
#         ],
#     'obspy.plugin.inventory.SHAPEFILE': [
#         'writeFormat = obspy.io.shapefile.core:_write_shapefile',
#         ],
#     'obspy.plugin.inventory.STATIONTXT': [
#         'isFormat = obspy.io.stationtxt.core:is_fdsn_station_text_file',
#         'readFormat = '
#         'obspy.io.stationtxt.core:read_fdsn_station_text_file',
#         'writeFormat = obspy.io.stationtxt.core:_write_stationtxt',
#         ],
#     'obspy.plugin.inventory.KML': [
#         'writeFormat = obspy.io.kml.core:_write_kml',
#         ],
#     'obspy.plugin.inventory.SEED': [
#         'isFormat = obspy.io.xseed.core:_is_seed',
#         'readFormat = obspy.io.xseed.core:_read_seed',
#     ],
#     'obspy.plugin.inventory.XSEED': [
#         'isFormat = obspy.io.xseed.core:_is_xseed',
#         'readFormat = obspy.io.xseed.core:_read_xseed',
#     ],
#     'obspy.plugin.inventory.RESP': [
#         'isFormat = obspy.io.xseed.core:_is_resp',
#         'readFormat = obspy.io.xseed.core:_read_resp',
#     ],
#     'obspy.plugin.detrend': [
#         'linear = scipy.signal:detrend',
#         'constant = scipy.signal:detrend',
#         'demean = scipy.signal:detrend',
#         'simple = obspy.signal.detrend:simple',
#         'polynomial = obspy.signal.detrend:polynomial',
#         'spline = obspy.signal.detrend:spline'
#         ],
#     'obspy.plugin.differentiate': [
#         'gradient = numpy:gradient',
#         ],
#     'obspy.plugin.integrate': [
#         'cumtrapz = '
#         'obspy.signal.differentiate_and_integrate:integrate_cumtrapz',
#         'spline = '
#         'obspy.signal.differentiate_and_integrate:integrate_spline',
#         ],
#     'obspy.plugin.filter': [
#         'bandpass = obspy.signal.filter:bandpass',
#         'bandstop = obspy.signal.filter:bandstop',
#         'lowpass = obspy.signal.filter:lowpass',
#         'highpass = obspy.signal.filter:highpass',
#         'lowpass_cheby_2 = obspy.signal.filter:lowpass_cheby_2',
#         'lowpass_fir = obspy.signal.filter:lowpass_FIR',
#         'remez_fir = obspy.signal.filter:remez_FIR',
#         ],
#     'obspy.plugin.interpolate': [
#         'interpolate_1d = obspy.signal.interpolation:interpolate_1d',
#         'weighted_average_slopes = '
#         'obspy.signal.interpolation:weighted_average_slopes',
#         'lanczos = obspy.signal.interpolation:lanczos_interpolation'
#         ],
#     'obspy.plugin.rotate': [
#         'rotate_ne_rt = obspy.signal.rotate:rotate_ne_rt',
#         'rotate_rt_ne = obspy.signal.rotate:rotate_rt_ne',
#         'rotate_zne_lqt = obspy.signal.rotate:rotate_zne_lqt',
#         'rotate_lqt_zne = obspy.signal.rotate:rotate_lqt_zne'
#         ],
#     'obspy.plugin.taper': [
#         'cosine = obspy.signal.invsim:cosine_taper',
#         'barthann = scipy.signal:barthann',
#         'bartlett = scipy.signal:bartlett',
#         'blackman = scipy.signal:blackman',
#         'blackmanharris = scipy.signal:blackmanharris',
#         'bohman = scipy.signal:bohman',
#         'boxcar = scipy.signal:boxcar',
#         'chebwin = scipy.signal:chebwin',
#         'flattop = scipy.signal:flattop',
#         'gaussian = scipy.signal:gaussian',
#         'general_gaussian = scipy.signal:general_gaussian',
#         'hamming = scipy.signal:hamming',
#         'hann = scipy.signal:hann',
#         'kaiser = scipy.signal:kaiser',
#         'nuttall = scipy.signal:nuttall',
#         'parzen = scipy.signal:parzen',
#         'slepian = scipy.signal:slepian',
#         'triang = scipy.signal:triang',
#         ],
#     'obspy.plugin.trigger': [
#         'recstalta = obspy.signal.trigger:recursive_sta_lta',
#         'carlstatrig = obspy.signal.trigger:carl_sta_trig',
#         'classicstalta = obspy.signal.trigger:classic_sta_lta',
#         'delayedstalta = obspy.signal.trigger:delayed_sta_lta',
#         'zdetect = obspy.signal.trigger:z_detect',
#         'recstaltapy = obspy.signal.trigger:recursive_sta_lta_py',
#         'classicstaltapy = obspy.signal.trigger:classic_sta_lta_py',
#         ],
#     'obspy.db.feature': [
#         'minmax_amplitude = obspy.db.feature:MinMaxAmplitudeFeature',
#         'bandpass_preview = obspy.db.feature:BandpassPreviewFeature',
#         ],
    }

# check for MSVC
if platform.system() == "Windows" and (
        'msvc' in sys.argv or
        '-c' not in sys.argv and
        get_default_compiler() == 'msvc'):
    IS_MSVC = True
else:
    IS_MSVC = False

# Use system libraries? Set later...
EXTERNAL_EVALRESP = False
EXTERNAL_LIBMSEED = False

# helper function for collecting export symbols from .def files
def export_symbols(*path):
    lines = open(os.path.join(*path), 'r').readlines()[2:]
    return [s.strip() for s in lines if s.strip() != '']

def find_packages():
    """
    Simple function to find all modules under the current folder.
    """
    modules = []
    for dirpath, _, filenames in os.walk(os.path.join(SETUP_DIRECTORY,
                                                      "obsln")):
        if "__init__.py" in filenames:
            modules.append(os.path.relpath(dirpath, SETUP_DIRECTORY))
    return [_i.replace(os.sep, ".") for _i in modules]

# adds --with-system-libs command-line option if possible

def add_features():
    if 'setuptools' not in sys.modules or not hasattr(setuptools, 'Feature'):
        return {}

    class ExternalLibFeature(setuptools.Feature):
        def __init__(self, *args, **kwargs):
            self.name = kwargs['name']
            setuptools.Feature.__init__(self, *args, **kwargs)

        def include_in(self, dist):
            globals()[self.name] = True

        def exclude_from(self, dist):
            globals()[self.name] = False

    return {
        'system-libs': setuptools.Feature(
            'use of system C libraries',
            standard=False,
            require_features=('system-evalresp', 'system-libmseed')
        ),
        'system-evalresp': ExternalLibFeature(
            'use of system evalresp library',
            standard=False,
            name='EXTERNAL_EVALRESP'
        ),
        'system-libmseed': ExternalLibFeature(
            'use of system libmseed library',
            standard=False,
            name='EXTERNAL_LIBMSEED'
        )
    }

def add_data_files(config):
    """
    Recursively include all non python files
    """
    # python files are included per default, we only include data files
    # here
    EXCLUDE_WILDCARDS = ['*.py', '*.pyc', '*.pyo', '*.pdf', '.git*']
    EXCLUDE_DIRS = ['src', '__pycache__']
    common_prefix = SETUP_DIRECTORY + os.path.sep
    for root, dirs, files in os.walk(os.path.join(SETUP_DIRECTORY, 'obsln')):
        root = root.replace(common_prefix, '')
        for name in files:
            if any(fnmatch.fnmatch(name, w) for w in EXCLUDE_WILDCARDS):
                continue
            config.add_data_files(os.path.join(root, name))
        for folder in EXCLUDE_DIRS:
            if folder in dirs:
                dirs.remove(folder)

    # Force include the contents of some directories.
    FORCE_INCLUDE_DIRS = [
        os.path.join(SETUP_DIRECTORY, 'obsln', 'io', 'mseed', 'src',
                     'libmseed', 'test')]

    for folder in FORCE_INCLUDE_DIRS:
        for root, _, files in os.walk(folder):
            for filename in files:
                config.add_data_files(
                    os.path.relpath(os.path.join(root, filename),
                                    SETUP_DIRECTORY))

def configuration(parent_package="", top_path=None):
    """
    Config function mainly used to compile C code.
    """
    config = Configuration("", parent_package, top_path)

    # GSE2
#     path = os.path.join("obsln", "io", "gse2", "src", "GSE_UTI")
#     files = [os.path.join(path, "gse_functions.c")]
#     # compiler specific options
#     kwargs = {}
#     if IS_MSVC:
#         # get export symbols
#         kwargs['export_symbols'] = export_symbols(path, 'gse_functions.def')
#     config.add_extension(_get_lib_name("gse2", add_extension_suffix=False),
#                          files, **kwargs)

    # LIBMSEED
#     path = os.path.join("obsln", "io", "mseed", "src")
#     files = [os.path.join(path, "obspy-readbuffer.c")]
#     if not EXTERNAL_LIBMSEED:
#         files += glob.glob(os.path.join(path, "libmseed", "*.c"))
#     # compiler specific options
#     kwargs = {}
#     if IS_MSVC:
#         # needed by libmseed lmplatform.h
#         kwargs['define_macros'] = [('WIN32', '1')]
#         # get export symbols
#         kwargs['export_symbols'] = \
#             export_symbols(path, 'libmseed', 'libmseed.def')
#         kwargs['export_symbols'] += \
#             export_symbols(path, 'obspy-readbuffer.def')
#     if EXTERNAL_LIBMSEED:
#         kwargs['libraries'] = ['mseed']
#     config.add_extension(_get_lib_name("mseed", add_extension_suffix=False),
#                          files, **kwargs)

    # SEGY
    path = os.path.join("obsln", "io", "segy", "src")
    files = [os.path.join(path, "ibm2ieee.c")]
    # compiler specific options
    kwargs = {}
    if IS_MSVC:
        # get export symbols
        kwargs['export_symbols'] = export_symbols(path, 'libsegy.def')
    config.add_extension(_get_lib_name("segy", add_extension_suffix=False),
                         files, **kwargs)

    # SIGNAL
#     path = os.path.join("obspy", "signal", "src")
#     files = glob.glob(os.path.join(path, "*.c"))
#     # compiler specific options
#     kwargs = {}
#     if IS_MSVC:
#         # get export symbols
#         kwargs['export_symbols'] = export_symbols(path, 'libsignal.def')
#     config.add_extension(_get_lib_name("signal", add_extension_suffix=False),
#                          files, **kwargs)

    # EVALRESP
#     path = os.path.join("obspy", "signal", "src")
#     if EXTERNAL_EVALRESP:
#         files = glob.glob(os.path.join(path, "evalresp", "_obspy*.c"))
#     else:
#         files = glob.glob(os.path.join(path, "evalresp", "*.c"))
#     # compiler specific options
#     kwargs = {}
#     if IS_MSVC:
#         # needed by evalresp evresp.h
#         kwargs['define_macros'] = [('WIN32', '1')]
#         # get export symbols
#         kwargs['export_symbols'] = export_symbols(path, 'libevresp.def')
#     if EXTERNAL_EVALRESP:
#         kwargs['libraries'] = ['evresp']
#     config.add_extension(_get_lib_name("evresp", add_extension_suffix=False),
#                          files, **kwargs)

    # TAU
#     path = os.path.join("obsln", "taup", "src")
#     files = [os.path.join(path, "inner_tau_loops.c")]
#     # compiler specific options
#     kwargs = {}
#     if IS_MSVC:
#         # get export symbols
#         kwargs['export_symbols'] = export_symbols(path, 'libtau.def')
#     config.add_extension(_get_lib_name("tau", add_extension_suffix=False),
#                          files, **kwargs)

    add_data_files(config)

    return config

def setupPackage():
    
    setup(
        name='obsln',
        description=DOCSTRING[1],
        long_description="\n".join(DOCSTRING[3:]),
        author='Bateared Collie',
        author_email='batearedcollie@gmail.com',
        packages=find_packages(),
        namespace_packages=[],
        zip_safe=False,
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRAS_REQUIRE,
        features=add_features(),
#         # this is needed for "easy_install obspy==dev"
#         download_url=("https://github.com/obspy/obspy/zipball/master"
#                       "#egg=obspy=dev"),
#         include_package_data=True,
        entry_points=ENTRY_POINTS,
        ext_package='obsln.lib',
#         cmdclass={
#             'build_man': Help2ManBuild,
#             'install_man': Help2ManInstall
#         },
        configuration=configuration
        )  


if __name__ == '__main__':
    # clean --all does not remove extensions automatically
    if 'clean' in sys.argv and '--all' in sys.argv:
        import shutil
        # delete complete build directory
        path = os.path.join(SETUP_DIRECTORY, 'build')
        try:
            shutil.rmtree(path)
        except Exception:
            pass
        # delete all shared libs from lib directory
        path = os.path.join(SETUP_DIRECTORY, 'obsln', 'lib')
        for filename in glob.glob(path + os.sep + '*.pyd'):
            try:
                os.remove(filename)
            except Exception:
                pass
        for filename in glob.glob(path + os.sep + '*.so'):
            try:
                os.remove(filename)
            except Exception:
                pass
        path = os.path.join(SETUP_DIRECTORY, 'obsln', 'taup', 'data', 'models')
        try:
            shutil.rmtree(path)
        except Exception:
            pass
    else:
        setupPackage()